@@ -16,30 +16,47 @@
 
 """bzr library"""
 
-from inventory import Inventory, InventoryEntry
-from branch import Branch, ScratchBranch
-from osutils import format_date
-from tree import Tree
-from diff import diff_trees
-from trace import mutter, warning
-import add
-
 BZRDIR = ".bzr"
 
 DEFAULT_IGNORE = ['.bzr.log',
                   '*~', '#*#', '*$', '.#*',
+                  '.*.sw[nop]', '.*.tmp',
                   '*.tmp', '*.bak', '*.BAK', '*.orig',
                   '*.o', '*.obj', '*.a', '*.py[oc]', '*.so', '*.exe', '*.elc', 
-                  '{arch}', 'CVS', '.svn', '_darcs', 'SCCS', 'RCS',
+                  '{arch}', 'CVS', 'CVS.adm', '.svn', '_darcs', 'SCCS', 'RCS',
+                  '*,v',
                   'BitKeeper',
-                  'TAGS', '.make.state', '.sconsign', '.tmp*']
+                  '.git',
+                  'TAGS', '.make.state', '.sconsign', '.tmp*',
+                  '.del-*',
+                  '.DS_Store',]
 
 IGNORE_FILENAME = ".bzrignore"
 
+import os
 import locale
-user_encoding = locale.getpreferredencoding()
+user_encoding = locale.getpreferredencoding() or 'ascii'
+del locale
 
 __copyright__ = "Copyright 2005 Canonical Development Ltd."
-__author__ = "Martin Pool <mbp@canonical.com>"
-__version__ = '0.0.4'
+__version__ = '0.1.1'
 
+
+def get_bzr_revision():
+    """If bzr is run from a branch, return (revno,revid) or None"""
+    import bzrlib.errors
+    from bzrlib.branch import Branch
+    
+    try:
+        branch = Branch.open(os.path.dirname(__path__[0]))
+        rh = branch.revision_history()
+        if rh:
+            return len(rh), rh[-1]
+        else:
+            return None
+    except bzrlib.errors.BzrError:
+        return None
+    
+def test_suite():
+    import selftest
+    return selftest.test_suite()
