@@ -16,15 +16,22 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from sets import Set
 import time
 
-import bzrlib
-from osutils import format_date
+from bzrlib.osutils import format_date
+
+
+def _countiter(it):
+    # surely there's a builtin for this?
+    i = 0
+    for j in it:
+        i += 1
+    return i        
+
+
 
 def show_info(b):
-    # TODO: Maybe show space used by working tree, versioned files,
-    # unknown files, text store.
+    import diff
     
     print 'branch format:', b.controlfile('branch-format', 'r').readline().rstrip('\n')
 
@@ -38,31 +45,45 @@ def show_info(b):
 
     count_version_dirs = 0
 
-    count_status = {'A': 0, 'D': 0, 'M': 0, 'R': 0, '?': 0, 'I': 0, '.': 0}
-    for st_tup in bzrlib.diff_trees(b.basis_tree(), b.working_tree()):
-        fs = st_tup[0]
-        count_status[fs] += 1
-        if fs not in ['I', '?'] and st_tup[4] == 'directory':
-            count_version_dirs += 1
-
+    basis = b.basis_tree()
+    working = b.working_tree()
+    work_inv = working.inventory
+    delta = diff.compare_trees(basis, working, want_unchanged=True)
+    
     print
     print 'in the working tree:'
-    for name, fs in (('unchanged', '.'),
-                     ('modified', 'M'), ('added', 'A'), ('removed', 'D'),
-                     ('renamed', 'R'), ('unknown', '?'), ('ignored', 'I'),
-                     ):
-        print '  %8d %s' % (count_status[fs], name)
-    print '  %8d versioned subdirector%s' % (count_version_dirs,
-                                             plural(count_version_dirs, 'y', 'ies'))
+    print '  %8s unchanged' % len(delta.unchanged)
+    print '  %8d modified' % len(delta.modified)
+    print '  %8d added' % len(delta.added)
+    print '  %8d removed' % len(delta.removed)
+    print '  %8d renamed' % len(delta.renamed)
+
+    ignore_cnt = unknown_cnt = 0
+    for path in working.extras():
+        if working.is_ignored(path):
+            ignore_cnt += 1
+        else:
+            unknown_cnt += 1
+
+    print '  %8d unknown' % unknown_cnt
+    print '  %8d ignored' % ignore_cnt
+
+    dir_cnt = 0
+    for file_id in work_inv:
+        if work_inv.get_file_kind(file_id) == 'directory':
+            dir_cnt += 1
+    print '  %8d versioned %s' \
+          % (dir_cnt,
+             plural(dir_cnt, 'subdirectory', 'subdirectories'))
 
     print
     print 'branch history:'
     history = b.revision_history()
     revno = len(history)
     print '  %8d revision%s' % (revno, plural(revno))
-    committers = Set()
+    committers = {}
     for rev in history:
-        committers.add(b.get_revision(rev).committer)
+        committers[b.get_revision(rev).committer] = True
     print '  %8d committer%s' % (len(committers), plural(len(committers)))
     if revno > 0:
         firstrev = b.get_revision(history[0])
@@ -75,11 +96,11 @@ def show_info(b):
         print '  latest revision: %s' % format_date(lastrev.timestamp,
                                                     lastrev.timezone)
 
-    print
-    print 'text store:'
-    c, t = b.text_store.total_size()
-    print '  %8d file texts' % c
-    print '  %8d kB' % (t/1024)
+#     print
+#     print 'text store:'
+#     c, t = b.text_store.total_size()
+#     print '  %8d file texts' % c
+#     print '  %8d kB' % (t/1024)
 
     print
     print 'revision store:'
@@ -88,9 +109,9 @@ def show_info(b):
     print '  %8d kB' % (t/1024)
 
 
-    print
-    print 'inventory store:'
-    c, t = b.inventory_store.total_size()
-    print '  %8d inventories' % c
-    print '  %8d kB' % (t/1024)
+#     print
+#     print 'inventory store:'
+#     c, t = b.inventory_store.total_size()
+#     print '  %8d inventories' % c
+#     print '  %8d kB' % (t/1024)
 
